! Use emacs f90-mode:   -*- mode:f90  -*-
PROGRAM CHK_DATA

!-------------------------------------------------------------------------------
! Simple program to dump the first few elements of the data array for each
! record of an ARL packed meteorological file. Used for diagnostic testing.
! Created: 23 Nov 1999 (RRD)
!          14 Dec 2000 (RRD) - fortran90 upgrade
!          18 Oct 2001 (RRD) - expanded grid domain
!
! $Log: chk_data.f,v $
! Revision 1.9  2007/04/17 17:40:12  trn
! Added header decoding of heights
!
! Revision 1.8  2007/04/02 17:45:42  trn
! More robust error handling
!
! Revision 1.7  2006/11/16 18:59:11  trn
! Changes for decoding fcst minutes from extended header
!
! Revision 1.6  2006/10/06 14:53:49  trn
! Added option for 16-bit files, revert back to using default reals
!
! Revision 1.5  2006/10/05 18:28:38  trn
! Use real*8 and added code from pakinp.f90 to make it more similar to
! hymodelc code unpacking
!
!-------------------------------------------------------------------------------

  implicit none

  REAL,          ALLOCATABLE :: RDATA(:,:)   
  CHARACTER(1),  ALLOCATABLE :: CPACK(:)

  CHARACTER(4)               :: KVAR, MODEL 
  CHARACTER(50)              :: LABEL          
  CHARACTER(256)             :: FILE
  integer, parameter :: maxlenh=6074
  CHARACTER(maxlenh)            :: HEADER
  LOGICAL                    :: FTEST

  INTEGER  :: NEXP,i1,i2,di,j1,j2,dj, kret, k, krec, IY,IM,ID,IH,IF,KL
  integer :: IYR,IMO,IDA,IHR,IFC,nxy,len
  integer ::          ICX,       MN,                                              &
         NX,       NY,        NZ,                                              &
         K_FLAG,   LENH

  integer :: ierr, itime, icontrol, tmpctrl

  real ::  PREC,VAR1,                                                 &
         POLE_LAT, POLE_LON,  REF_LAT,                                         &
         REF_LON,  SIZE,      ORIENT,                                          &
         TANG_LAT, SYNC_XP,   SYNC_YP,                                         &
         SYNC_LAT, SYNC_LON,  DUMMY

  integer :: iroot,ivar,nvars
  integer, allocatable :: outunits(:)
  character (len=4), allocatable :: outvars(:)
  character (len=256) :: outfname
  real :: xi_coord, xj_coord
  logical :: use_16
  integer :: kol, l, num_varb
  real :: height

!-------------------------------------------------------------------------------
  INTERFACE
  SUBROUTINE UNPACK(label,icx,mn,CPACK,RDATA,NX,NY,NEXP,PREC,VAR1,i1,i2,di,j1,j2,dj, &
       &            outunits,outvars,nvars,IY,IM,ID,IH,IF,KL,KVAR,xi_coord, xj_coord,use_16)
  CHARACTER(50),INTENT(IN)  :: label
  CHARACTER(1),INTENT(IN)  :: CPACK(:)  
  INTEGER,     INTENT(IN)  :: NX,NY,NEXP,i1,i2,di,j1,j2,dj,icx,mn
  REAL,        INTENT(OUT) :: RDATA(NX,NY)   
  REAL,        INTENT(IN)  :: PREC,VAR1
  character (len=4), intent(in) :: outvars(:),KVAR
  integer, intent(in) ::      outunits(:),nvars,IY,IM,ID,IH,IF,KL
  real, intent(in) :: xi_coord, xj_coord
  logical, intent (in) :: use_16
  END SUBROUTINE
  END INTERFACE
!-------------------------------------------------------------------------------

  i1=1
  di=50
  i2=-1
  j1=1
  dj=50
  j2=-1
  icontrol = 0
  use_16 = .TRUE.
! directory and file name
  WRITE(*,*)'Enter file name (absolute path):'
  READ(*,'(a)') FILE
  WRITE(*,*)'Enter T/F for whether this is a 16-bit file (RAMS, DWRF, ECX):'
  READ(*,*) use_16
  WRITE(*,'(a)')'Enter type of run:',' 0 - interactive',' 1 - time inventory only ',&
       & ' 2 - variables and levels for first time period', &
       & ' 3 - also min,max,mean,sample values, for first time period only', &
       & ' 4 - everything '
  READ(*,*) icontrol
  if (icontrol .ge. 3) then
     WRITE(*,*)'Enter i and j loop limits (first, last, increment) for printouts',&
          & '(first <0:suppress printout; last<0: go to end of array size)'
     READ(*,*) i1,i2,di,j1,j2,dj
  endif
  nvars = 0
  allocate(outvars(1),outunits(1))
  outvars(:)=''
  outunits(:)=-99
  if (icontrol .ge. 4) then
     write (*,*) 'Enter number of output variables to be interpolated'
     READ(*,*,err=10,end=10) nvars
     if (nvars .ge. 1) then
        write (*,*) 'Enter xi,xj output coordinates:'
        read (*,*) xi_coord, xj_coord
        deallocate(outvars,outunits)
        allocate(outvars(nvars),outunits(nvars))
        iroot=index(file,'/',back=.true.) !zero if no '/'
        outunits(1)=20
        do ivar=1,nvars
           write (*,*) 'Enter 4-character variable name:'
           read (*,'(a)') outvars(ivar)
           if (ivar .gt. 1) outunits(ivar)=outunits(ivar-1)+1
           outfname = trim(file(iroot+1:))//'_'//outvars(ivar)
           write (*,'(a,i4,2a)') 'Writing variable no',ivar,' to ',trim(outfname)
           open(unit=outunits(ivar),file=outfname)
           write (outunits(ivar),'(a,/,2e20.10,/,6a5,a20)') &
                & trim(file),xi_coord, xj_coord, &
                & 'yr','mo','da','hr','fhr','lvl',outvars(ivar)
        end do
     end if
  end if
  
10 continue

! test for meteo file existence
  INQUIRE(FILE=FILE,EXIST=FTEST)
  IF(.NOT.FTEST)THEN
     WRITE(*,*)'Unable to find file: ',trim(FILE)
     STOP
  END IF

! open file to decode the standard label (50) plus the
! fixed portion (108) of the extended header
  OPEN(10,FILE=FILE,RECL=158,ACCESS='DIRECT',FORM='UNFORMATTED')

! decode the standard portion of the index record
  READ(10,REC=1)LABEL,HEADER(1:108)
  READ(LABEL,'(5I2,4X,A4)')IYR,IMO,IDA,IHR,IFC,KVAR
  WRITE(*,'(A,4I5)')'Opened file       : ',IYR,IMO,IDA,IHR

  IF(KVAR.NE.'INDX')THEN
     WRITE(*,*)'WARNING Old format meteo data grid'
     WRITE(*,*)LABEL
     WRITE(*,*)HEADER(1:108)
     STOP
  END IF

! decode extended portion of the header
  READ(HEADER(1:108),'(A4,I3,I2,12F7.0,3I3,I2,I4)',iostat=ierr)                &
         MODEL,    ICX,       MN,                                              &
         POLE_LAT, POLE_LON,  REF_LAT,                                         &
         REF_LON,  SIZE,      ORIENT,                                          &
         TANG_LAT, SYNC_XP,   SYNC_YP,                                         &
         SYNC_LAT, SYNC_LON,  DUMMY,                                           &
         NX,       NY,        NZ,                                              &
         K_FLAG,   LENH

  if (ierr .ne. 0) then
     WRITE(*,'(a)') 'ERROR: decoding first header, cannot recover'
     WRITE(*,'(2a)') 'LABEL=',LABEL
     WRITE(*,'(2a)') 'HEADER=',HEADER(1:108)
  end if

! close file and reopen with proper length
  CLOSE (10)
  NXY = NX*NY
  if (use_16) nxy=2*nxy
  LEN = NXY+50
  OPEN(10,FILE=FILE,RECL=LEN,ACCESS='DIRECT',FORM='UNFORMATTED')
  if (i1 .lt. 0) i1=nx+1
  if (j1 .lt. 0) j1=ny+1
  if (i2 .lt. 0) i2=nx
  if (j2 .lt. 0) j2=ny

! print file diagnostic
  WRITE(*,'(A,4I8)')'Grid size and lrec: ',NX,NY,NXY,LEN
  WRITE(*,'(A,I5)') 'Header record size: ',LENH

! allocate array space
  ALLOCATE (RDATA(NX,NY))   
  ALLOCATE (CPACK(NXY))

! read entire file and print headers
  KREC=1
  itime = 0
  tmpctrl=icontrol
100 READ(10,REC=KREC,ERR=800)LABEL,(CPACK(K),K=1,NXY)
    READ(LABEL,'(6I2,2X,A4,I4,2E14.7)',iostat=ierr) IY,IM,ID,IH,IF,KL,  &
                                             KVAR,NEXP,PREC,VAR1

    if (ierr .ne. 0) then
       WRITE(*,'(a)') 'ERROR: decoding LABEL, going on to next record'
       WRITE(*,'(2a)') 'LABEL=',LABEL
       WRITE(*,'(2a)') 'HEADER=',HEADER(1:108)
       krec=krec+1
       goto 100
    end if

    if (kvar .eq. 'INDX' .or. itime .eq. 0) then
       if (lenh .gt. maxlenh .or. lenh .gt. nxy) then
          write (*,*) 'Warning: ignoring header characters from ',min(maxlenh,nxy), &
               & ' to ',lenh
          lenh=min(maxlenh,nxy)
       end if
       do k=1,lenh
          header(k:k) = cpack(k)
       enddo
       WRITE(*,'(A,1x,a)')LABEL,header(1:108)
! decode extended portion of the header
       READ(HEADER(1:108),'(A4,I3,I2,12F7.0,3I3,I2,I4)',iostat=ierr)              &
            MODEL,    ICX,       MN,                                              &
            POLE_LAT, POLE_LON,  REF_LAT,                                         &
            REF_LON,  SIZE,      ORIENT,                                          &
            TANG_LAT, SYNC_XP,   SYNC_YP,                                         &
            SYNC_LAT, SYNC_LON,  DUMMY,                                           &
            NX,       NY,        NZ,                                              &
            K_FLAG,   LENH
       if (ierr .ne. 0) then
          WRITE(*,'(a)') 'ERROR: decoding HEADER'
          WRITE(*,'(2a)') 'LABEL=',LABEL
          WRITE(*,'(2a)') 'HEADER=',HEADER(1:108)
       end if
       if (itime .eq. 0) then
          !this code taken from metset:
          kol=109
          height_loop: do l=1,nz
             READ(HEADER(KOL:KOL+7),'(F6.2,I2)',iostat=ierr) HEIGHT, NUM_VARB
             if (ierr .ne. 0) then
                write (*,*) 'Error decoding level ',l,' kol=',kol
                exit height_loop
             end if
             write (*,'(a,i4,a,g15.6,a,i4)') 'INDX: l=',l,' height=',height, &
                  & ' num_varb=',num_varb
             KOL=KOL+8
             do k=1,num_varb
                if (use_16) then
                   KOL=KOL+10
                else
                   KOL=KOL+8
                end if
             end do
          end do height_loop
       endif
       itime = itime + 1
       if (itime .gt. 1 .and. icontrol .lt. 4) tmpctrl=1
    else
       if (icontrol .eq. 0) then
          READ(*,*,END=800)
          CALL UNPACK(label,icx,mn,CPACK,RDATA,NX,NY,NEXP,PREC,VAR1,i1,i2,di,j1,j2,dj, &
       &            outunits,outvars,nvars,IY,IM,ID,IH,IF,KL,KVAR,xi_coord, xj_coord,use_16)
       else
          if (tmpctrl .ge. 2) WRITE(*,'(A)')LABEL
          if (tmpctrl .ge. 3) CALL UNPACK(label,icx,mn,CPACK,RDATA,NX,NY,NEXP,PREC,VAR1, &
               &    i1,i2,di,j1,j2,dj, &
               &    outunits,outvars,nvars,IY,IM,ID,IH,IF,KL,KVAR,xi_coord, xj_coord,use_16)
       endif
    endif

    KREC=KREC+1
  GO TO 100

800 continue
  do ivar=1,nvars
     close(outunits(ivar))
  end do

  STOP

900 WRITE(*,*)'ERROR: decoding header'
    WRITE(*,*)LABEL
    WRITE(*,*)HEADER(1:108)

END PROGRAM chk_data

!-------------------------------------------------------------------------------

SUBROUTINE UNPACK(label,icx,mn,CPACK,RDATA,NX,NY,NEXP,PREC,VAR1,i1,i2,di,j1,j2,dj, &
       &            outunits,outvars,nvars,IY,IM,ID,IH,IF,KL,KVAR,xi_coord, xj_coord,use_16)

  implicit none

  CHARACTER(50),INTENT(IN)  :: label
  CHARACTER(1),INTENT(IN)  :: CPACK(:)  
  INTEGER,     INTENT(IN)  :: NX,NY,NEXP,i1,i2,di,j1,j2,dj,icx,mn
  REAL,        INTENT(OUT) :: RDATA(NX,NY)   
  REAL,        INTENT(IN)  :: PREC,VAR1
  character (len=4), intent(in) :: outvars(:),KVAR
  integer, intent(in) ::      outunits(:),nvars,IY,IM,ID,IH,IF,KL
  real, intent(in) :: xi_coord, xj_coord
  logical, intent (in) :: use_16

  real ::rmin, rmax, rmean

  real :: scale, vold, rmean_i, rmin_i, rmax_i
  integer :: indx, tstj, j, tsti, i, itmp, itmp1, itmp2

  integer :: ivar, ivarout
  integer :: xi1, xi2, xj1, xj2
  real :: xout, wt_i, wt_j

! only required when dealing with SUN F90 compiler
! replace ICHAR below with internally defined JCHAR function
! CHARACTER MYCHR*1
! JCHAR(MYCHR)=IAND(ICHAR(MYCHR),255)

  if (.not. use_16) then
     SCALE=2.0**(7-NEXP)
  else
     scale=2.0**(15-NEXP)
  end if
  
  VOLD=VAR1
  INDX=0
  tstj=j1
  rmean = 0.
  rmean_i = 0.
  rmin_i = +1.e10
  rmax_i = -1.e10
  DO J=1,NY
     tsti=i1
     DO I=1,NX
        INDX=INDX+1
! the following 2 lines taken from pakinp.f90:
        if (.not. use_16) then
           ITMP=ICHAR(CPACK(INDX))
           IF(ITMP.LT.0)ITMP=ITMP+256
           RDATA(I,J)=(ITMP-127.)/SCALE+VOLD
        else
           ITMP1=ICHAR(CPACK(INDX))
           INDX=INDX+1
           ITMP2=ICHAR(CPACK(INDX))
           ITMP=ITMP2*256+ITMP1
           IF(ITMP.LT.0)ITMP=ITMP+65536
           RDATA(I,J)=((ITMP-32767.0)/SCALE)+VOLD
        end if
        
        VOLD=RDATA(I,J)
! the following line taken from pakinp.f90:
        IF(ABS(RDATA(I,J)) .LT. PREC) RDATA(I,J)=0.0
        rmean = rmean + rdata(i,j)
        if (i .le. nx-1 .and. j .le. ny-1) then
           rmean_i=rmean_i+rdata(i,j)
           rmin_i=min(rmin_i,rdata(i,j))
           rmax_i=max(rmax_i,rdata(i,j))
        endif
        IF(I.eq.tsti.AND.J.eq.tstj) then
           WRITE(*,'(3I5,E15.6)')J,I,ICHAR(CPACK(INDX)),RDATA(I,J)
           tsti = tsti + di
           if (tsti .gt. i2) tsti=-99
        endif
     END DO
     IF(J.eq.tstj) then
        tstj=tstj+dj
        if (tstj .gt. j2) tstj=-99
     endif
     VOLD=RDATA(1,J)
  END DO
  rmin = minval(rdata)
  rmax = maxval(rdata)
  rmean = rmean/(nx*ny)
  rmean_i = rmean_i/((nx-1.)*(ny-1.))
  WRITE(*,'(a,2i4,a,6g15.6)') label(1:18),icx,mn,': min,max,mean=',rmin,rmax,rmean, &
       & rmin_i,rmax_i,rmean_i
  
  if (nvars .ge. 1) then
     ivarout=0
     do ivar=1,nvars
        if (kvar .eq. outvars(ivar)) ivarout=ivar
     end do
     if (ivarout .gt. 0) then
        xi1=max(1,min(int(xi_coord),nx))
        wt_i=xi_coord - xi1
        xi2=min(xi1+1,nx)
        xj1=max(1,min(int(xj_coord),ny))
        wt_j=xj_coord - xj1
        xj2=min(xj1+1,ny)
        xout = (1.-wt_i)*(1.-wt_j)*rdata(xi1,xj1) + &
             &     wt_i *(1.-wt_j)*rdata(xi2,xj1) + &
             & (1.-wt_i)*    wt_j *rdata(xi1,xj2) + &
             &     wt_i *    wt_j *rdata(xi2,xj2)
        write (outunits(ivarout),'(6i5,e20.10)') IY,IM,ID,IH,IF,KL,xout
     end if
  end if
  
  return

END SUBROUTINE unpack
