#!/bin/csh -f

switch ($#argv)
case 1:
case 0:
	echo "$0 aborting because supplied $#argv arguments"
	echo "It needs 2 or more:"
	echo "1 - ndomains"
	echo "2 or more - idays (yyyymmdd)"
        echo "Environment variables used for overriding defaults:"
        echo "  RESTORED_ARL - parent directory of arl/yyyymmdd working directories and Metfiles directory"
        echo "  METFILES - destination directory of dnn.yyyymmdd.arl files, defaults to RESTORED_ARL/Metfiles"
	exit (1)
default:
        set ndomains = $argv[1]
	shift
endsw

if ($?RESTORED_ARL) then
 set arldir=${RESTORED_ARL}/arl
else
 echo "Missing required environment variables RESTORED_ARL"
 exit 1
endif
if ($?METFILES) then
 set metdir=$METFILES
else
 set metdir=${RESTORED_ARL}/Metfiles
endif

echo "Running $0 with:"
echo " ndomains=$ndomains"
echo " arldir=$arldir"
echo " metdir=$metdir"

set prevdir=""
foreach iday ( $* )
  @ idom=0
  while ($idom < $ndomains)
    @ idom++
    set domain="d$idom"
    if ($idom < 10) set domain="d0$idom"
    echo ""
    echo "Processing iday=$iday, domain=$domain"
    cd $arldir/$iday
    if ("a$prevdir" == "a") then
      echo "Not linking to previous iday forecast"
    else
      set prevfile=`/bin/ls -1 ../$prevdir/wrfout_${domain}_????-??-??_*.DATA_MASS.WRF | tail -1`
      echo "ln -s $prevfile ."
      ln -s $prevfile .
    endif
    if (-e ${metdir}/${domain}.${iday}.arl) then
      echo "Not creating ${metdir}/${domain}.${iday}.arl, it already exists"
    else
      echo "Creating ${metdir}/${domain}.${iday}.arl"
      cat wrfout_${domain}_????-??-??_*.DATA_MASS.WRF > ${metdir}/${domain}.${iday}.arl
    endif
  end
  set prevdir=$iday
end

